@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2016-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2016-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -129,11 +129,14 @@ public:
     /** Defines size adjust @a enmPolicy. */
     void setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy enmPolicy);
     /** Marks the line edit of the combobox. Refer to QILineEdit::mark(..). */
-    void mark(bool fError, const QString &strErrorMessage = QString());
+    void mark(bool fError, const QString &strErrorMessage, const QString &strNoErrorMessage);
 
     /** Inserts separator at position with specified @a iIndex. */
     void insertSeparator(int iIndex);
 
+    /** Calls QILineEdit member's setMarkable API. */
+    void setMarkable(bool fMarkable);
+
 public slots:
 
     /** Clears the combobox, removing all items. */
